# -*- coding:utf-8 -*-
#
#    Naive Bayes classifier for language identification v0.1
#    Copyright 2012 Nikola Ljubešić
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re

class naive_bayes_classifier():
  
  def __init__(self):
    self.distr={}
    self.parms={}

  def calculate_distr(self,text,distr):
    for token in re.findall(r'\w+',text.lower(),re.UNICODE):
    #for token in re.findall(r'\w+',text,re.UNICODE):
    #  if token.lower()==token:
      distr[token]=distr.get(token,0)+1

  def add_category(self,category,text):
    self.distr[category]={}
    self.calculate_distr(text,self.distr[category])

  def calculate_parms(self,strongest=None):
    from math import log
    features=set()
    categories=self.distr.keys()
    for category in categories:
      features=features.union(set(self.distr[category]))
      self.parms[category]={}
    for feature in features:
      feature_sum=0.0
      for category in categories:
        feature_sum+=self.distr[category].get(feature,0)+0.1
        self.parms[category][feature]=self.distr[category].get(feature,0)+0.1
      for category in categories:
        self.parms[category][feature]=log(self.parms[category][feature]/feature_sum,2)
    if strongest is not None:
      strongest_features={}
      for feature in features:
        weights=[self.parms[category][feature] for category in categories]
        strongest_features[feature]=max(weights)-min(weights)
      #print sorted(strongest_features.items(),reverse=True,key=lambda x:x[1])[:strongest]
      for feature,diff in sorted(strongest_features.items(),key=lambda x:x[1])[:-strongest]:
        for category in categories:
          del self.parms[category][feature]
    del self.distr
  
  def guess_category(self,text):
    distr={}
    self.calculate_distr(text,distr)
    categories=self.parms.keys()
    probabilities=dict([(category,0.0) for category in categories])
    for feature in distr:
      if feature not in self.parms[categories[0]]:
        continue
      for category in categories:
        probabilities[category]+=self.parms[category][feature]*distr[feature]
    first,second=sorted(probabilities.items(),reverse=True,key=lambda x:x[1])[:2]
    return first[0],second[0],second[1]/(first[1]+second[1])#,first,second

  def get_strongest_features(self,text,i=10):
    distr={}
    self.calculate_distr(text,distr)
    categories=self.parms.keys()
    for category in categories:
      features={}
      for feature in distr:
        if feature in self.parms[categories[0]]:
          features[feature]=self.parms[category][feature]*distr[feature]
      print category
      print sorted(features.items(),reverse=True,key=lambda x:x[1])[:i]

  @classmethod
  def read_classifier(cls,path):
    import cPickle as pickle
    return pickle.load(open(path))  

  def serialize_classifier(self,path):
    import cPickle as pickle
    pickle.dump(self,open(path,'w'),1)
    
    
if __name__=='__main__':
  import sys
  #
  #train
  #nb=naive_bayes_classifier()
  #nb.add_category('bs',open('bhs_bs.txt').read().decode('utf8'))
  #nb.add_category('hr',open('bhs_hr.txt').read().decode('utf8'))
  #nb.add_category('sr',open('bhs_sr_Latn.txt').read().decode('utf8'))
  #nb.calculate_parms()
  #nb.serialize_classifier('bs-hr-sr.classifier')
  #
  #strongest features
  #nb=naive_bayes_classifier.read_classifier(sys.argv[1])
  #for category in nb.parms:
  #  print category
  #  print sorted(nb.parms[category].items(),reverse=True,key=lambda x:x[1])[:100]
  #
  #predict
  if len(sys.argv)!=3:
    print 'Usage: python naive_bayes_classifier.py model text_file'
    print 'Example: python naive_bayes_classifier.py bs-hr-sr.classifier dnevniavaz.ba.txt'
    sys.exit(1)
  nb=naive_bayes_classifier.read_classifier(sys.argv[1])
  i=0
  for line in open(sys.argv[2]):
    i+=1
    print i,nb.guess_category(line.decode('utf8'))
